@@ -74,7 +74,8 @@ Note:
 """
 import sys
 import time
-from threading import Thread as BaseThread, Event, Timer, enumerate
+import threading
+import contextlib
 
 try:
     from queue import Queue, Empty
@@ -82,50 +83,33 @@ except ImportError:
     from Queue import Queue, Empty
 
 
-__all__ = ['Queue', 'Empty', 'Thread', 'ContinuousThread', 'PausableThread', 'OperationThread', 'PeriodicThread',
-           'shutdown']
+__all__ = [
+    'is_py27', 'Queue', 'Empty',
+    'Thread', 'ContinuousThread', 'PausableThread', 'OperationThread', 'PeriodicThread'
+    ]
 
 
-SMALL_SLEEP_VALUE = 0.0000000000001
+SMALL_SLEEP_VALUE = 0.00001
 is_py27 = sys.version_info < (3, 0)
 
+# Thread Objects
+BaseThread = threading.Thread
+Event = threading.Event
+Timer = threading.Timer
+RLock = threading.RLock
 
-def shutdown(timeout=None):
-    """Python's threading._shutdown is no longer called or hangs before exit.
-    This library depends on that function. That function found all of the non-daemon threads and
-    called the join method. This threading library was built around that function. This function
-    calls join(0) on all of the non-daemon threads to ensure cleanup from every thread.
 
-    You may want to put this at the end of your code. Atexit will not work for this.
-
-    I did not have a problem with Python 3.4. I noticed this issue in Python 3.8. I do not know
-    when or why this stopped working.
-
-    .. code-block :: python
-
-        import time
-        import continuous_threading
-
-        def do_something():
-            print('hello')
-            time.sleep(1)
-
-        th = continuous_threading.PausableThread(target=do_something)
-        th.start()
-
-        time.sleep(10)
-
-        continuous_threading.shutdown()
-
-    Args:
-        timeout (int/float)[None]: Timeout argument to pass into every thread's join method.
-    """
-    for th in enumerate():
-        try:
-            if th.is_alive() and not th.isDaemon():
-                th.join(timeout)
-        except:
-            pass
+@contextlib.contextmanager
+def safe_release(lock):
+    """Fixes RuntimeError for unlocking a released lock."""
+    # with lock:
+    #     yield
+    try:
+        lock.acquire()
+        yield
+    finally:
+        if lock.locked():
+            lock.release()
 
 
 class Thread(BaseThread):
@@ -140,14 +124,15 @@ class Thread(BaseThread):
         close properly. These threads are setup to handle closing a looping thread.
     """
 
-    def __init__(self, target=None, name=None, args=None, kwargs=None, daemon=None):
+    def __init__(self, target=None, name=None, args=None, kwargs=None, daemon=None, group=None):
         self.force_non_daemon = True
         self.close_warning = False
+        self._is_alive = Event()
         if args is None:
             args = tuple()
         if kwargs is None:
             kwargs = dict()
-        super(Thread, self).__init__(target=target, name=name, args=args, kwargs=kwargs)
+        super(Thread, self).__init__(target=target, name=name, args=args, kwargs=kwargs, daemon=daemon, group=group)
 
         # Check the daemon argument
         if daemon is not None:
@@ -172,12 +157,12 @@ class Thread(BaseThread):
             `start()` function is called. If you want to run a daemon thread set `force_non_daemon = False` and set
             `daemon = True`. If you do this then the `close()` function is not guaranteed to be called.
         """
-        global SHUTDOWN_REGISTERED
         if not self._started.is_set():
             # If daemone=False python forces join to be called which closes the thread properly.
             if self.force_non_daemon:
                 self.daemon = False
 
+            self._is_alive.set()
             super(Thread, self).start()
 
     def stop(self):
@@ -188,13 +173,44 @@ class Thread(BaseThread):
         """Close the thread (clean up variables)."""
         self.stop()
 
+    def is_alive(self):
+        """Return if the Thread is alive.
+
+        The threading.Thread.is_alive uses the "_tstate_lock". This library implemented allow_shutdown() which allows
+        users to change the "_tstate_lock". This stops threading._shutdown() from halting until "_tstate_lock"
+        is released.
+        """
+        try:
+            return self._is_alive.is_set()
+        except (AttributeError, Exception):
+            return False
+
     def _run(self, *args, **kwargs):
         """Default function target to run if a target is not given."""
         pass
+
+    def allow_shutdown(self):
+        """Release the "_tstate_lock" to allow threading._shutdown to finish.
+
+        The threading._shutdown() function waits for all non-daemon threads to release their "_t_state_lock".
+        This allows the ContinuousThread to join() automatically at exit.
+
+        Returns:
+            lock (RLock): The "_tstate_lock" or new RLock that can be used to safely run code blocks.
+                See Also ContinuousThread.run().
+        """
+        lock = getattr(self, '_tstate_lock')
+        if lock is not None:
+            lock.release()
+        else:
+            lock = RLock()
+        return lock
     
     def join(self, timeout=None):
         """Properly close the thread."""
         try:
+            self._is_alive.clear()
+
             # Close warning
             join_tmr = self._create_close_warning_timer(timeout)
 
@@ -249,10 +265,31 @@ class ContinuousThread(Thread):
     """Thread that is continuously running and closes properly. Do not override the run method.
     If you are using this class with inheritance override the '_run' method.
     """
-    def __init__(self, target=None, name=None, args=None, kwargs=None, daemon=None):
+    def __init__(self, target=None, name=None, args=None, kwargs=None, daemon=None, group=None,
+                 init=None, iargs=None, ikwargs=None):
+        """Initialize the thread object.
+
+        Args:
+            target (object)[None]: Target functions to run in a separate thread.
+            name (str)[None]: Name of the new process.
+            args (tuple)[None]: Default positional arguments to pass into the given target function.
+            kwargs (dict)[None]: Default keyword arguments to pass into the given target function.
+            daemon (bool)[None]: If this process should be a daemon process. This is automatically forced to be False.
+                Non-daemon process/threads call join when python exits.
+            group (object)[None]: Not used in python multiprocessing at this time.
+            init (callable)[None]: Run this function at the start of the process. If it returns a dictionary pass the
+                dictionary as keyword arguments into the target function.
+            iargs (tuple)[None]: Positional arguments to pass into init
+            ikwargs (dict)[None]: Keyword arguments to pass into init.
+        """
         # Thread properties
         self.alive = Event()  # If the thread is running
-        super(ContinuousThread, self).__init__(target=target, name=name, args=args, kwargs=kwargs, daemon=daemon)
+        self.init = init
+        self.iargs = iargs or tuple()
+        self.ikwargs = ikwargs or dict()
+
+        super(ContinuousThread, self).__init__(target=target, name=name, args=args, kwargs=kwargs,
+                                               daemon=daemon, group=group)
 
     def is_running(self):
         """Return if the serial port is connected and alive."""
@@ -274,13 +311,30 @@ class ContinuousThread(Thread):
         """Run method called if a target is not given to the thread. This method should be overridden if inherited."""
         pass
 
+    def run_init(self):
+        """Run the init function and return the args and kwargs."""
+        args = self._args
+        kwargs = self._kwargs
+        if callable(self.init):
+            kwds = self.init(*self.iargs, **self.ikwargs)
+            if isinstance(kwds, dict) and len(kwds) > 0:
+                kwds.update(kwargs)
+                kwargs = kwds
+        return args, kwargs
+
     def run(self):
         """The thread will loop through running the set _target method (default _run()). This 
         method can be paused and restarted.
         """
+        args, kwargs = self.run_init()
+
+        # Allow threading._shutdown() to continue
+        lock = self.allow_shutdown()
+
         while self.alive.is_set():
-            # Run the thread method
-            self._target(*self._args, **self._kwargs)
+            # Run the thread method while protected in the lock state
+            with safe_release(lock):
+                self._target(*args, **kwargs)
 # end class ContinuousThread
 
 
@@ -289,9 +343,26 @@ class PausableThread(ContinuousThread):
     the run method. If you are using this class with inheritance override the '_run' method.
     """
 
-    def __init__(self, target=None, name=None, args=None, kwargs=None, daemon=None):
+    def __init__(self, target=None, name=None, args=None, kwargs=None, daemon=None, group=None,
+                 init=None, iargs=None, ikwargs=None):
+        """Initialize the thread object.
+
+        Args:
+            target (object)[None]: Target functions to run in a separate thread.
+            name (str)[None]: Name of the new process.
+            args (tuple)[None]: Default positional arguments to pass into the given target function.
+            kwargs (dict)[None]: Default keyword arguments to pass into the given target function.
+            daemon (bool)[None]: If this process should be a daemon process. This is automatically forced to be False.
+                Non-daemon process/threads call join when python exits.
+            group (object)[None]: Not used in python multiprocessing at this time.
+            init (callable)[None]: Run this function at the start of the process. If it returns a dictionary pass the
+                dictionary as keyword arguments into the target function.
+            iargs (tuple)[None]: Positional arguments to pass into init
+            ikwargs (dict)[None]: Keyword arguments to pass into init.
+        """
         self.kill = Event()  # Loop condition to exit and kill the thread
-        super(PausableThread, self).__init__(target=target, name=name, args=args, kwargs=kwargs, daemon=daemon)
+        super(PausableThread, self).__init__(target=target, name=name, args=args, kwargs=kwargs,
+                                             daemon=daemon, group=group, init=init, iargs=iargs, ikwargs=ikwargs)
 
     def is_running(self):
         """Return if the serial port is connected and alive."""
@@ -333,13 +404,19 @@ class PausableThread(ContinuousThread):
         """The thread will loop through running the set _target method (default _run()). This 
         method can be paused and restarted.
         """
+        args, kwargs = self.run_init()
+
+        # Allow threading._shutdown() to continue
+        lock = self.allow_shutdown()
+
         while not self.kill.is_set():
             self.alive.wait()  # If alive is set then it does not wait according to docs.
             if self.kill.is_set():
                 break
 
-            # Run the read and write
-            self._target(*self._args, **self._kwargs)
+            # Run the thread method while protected in the lock state
+            with safe_release(lock):
+                self._target(*args, **kwargs)
         # end
 
         self.alive.clear()  # The thread is no longer running
@@ -353,10 +430,27 @@ class OperationThread(ContinuousThread):
     Data must be the first argument of the target function.
     """
 
-    def __init__(self, target=None, name=None, args=None, kwargs=None, daemon=None):
+    def __init__(self, target=None, name=None, args=None, kwargs=None, daemon=None, group=None,
+                 init=None, iargs=None, ikwargs=None):
+        """Initialize the thread object.
+
+        Args:
+            target (object)[None]: Target functions to run in a separate thread.
+            name (str)[None]: Name of the new process.
+            args (tuple)[None]: Default positional arguments to pass into the given target function.
+            kwargs (dict)[None]: Default keyword arguments to pass into the given target function.
+            daemon (bool)[None]: If this process should be a daemon process. This is automatically forced to be False.
+                Non-daemon process/threads call join when python exits.
+            group (object)[None]: Not used in python multiprocessing at this time.
+            init (callable)[None]: Run this function at the start of the process. If it returns a dictionary pass the
+                dictionary as keyword arguments into the target function.
+            iargs (tuple)[None]: Positional arguments to pass into init
+            ikwargs (dict)[None]: Keyword arguments to pass into init.
+        """
         self._operations = Queue()
         self.stop_processing = False
-        super(OperationThread, self).__init__(target=target, name=name, args=args, kwargs=kwargs, daemon=daemon)
+        super(OperationThread, self).__init__(target=target, name=name, args=args, kwargs=kwargs,
+                                              daemon=daemon, group=group, init=init, iargs=iargs, ikwargs=ikwargs)
 
     def get_queue_size(self):
         """Return the operation Queue size."""
@@ -371,17 +465,24 @@ class OperationThread(ContinuousThread):
         """The thread will loop through running the set _target method (default _run()). This 
         method can be paused and restarted.
         """
+        args, kwargs = self.run_init()
+
+        # Allow threading._shutdown() to continue
+        lock = self.allow_shutdown()
+
         while self.alive.is_set():
             try:
                 # Wait for data and other arguments
-                args, kwargs = self._operations.get(timeout=1)
+                op_args, op_kwargs = self._operations.get(timeout=1)
 
                 # Check if this data should be executed
                 if not self.stop_processing:
-                    # Run the data through the target function
-                    args = args or self._args
-                    kwargs = kwargs or self._kwargs
-                    self._target(*args, **kwargs)
+                    # Run the thread method while protected in the lock state
+                    with safe_release(lock):
+                        # Run the data through the target function
+                        op_args = op_args or args
+                        op_kwargs.update(kwargs)
+                        self._target(*op_args, **op_kwargs)
             except Empty:
                 continue
 
@@ -390,23 +491,42 @@ class OperationThread(ContinuousThread):
 
 
 class PeriodicThread(ContinuousThread):
-    def __init__(self, interval, target=None, name=None, args=None, kwargs=None, daemon=None):
+    def __init__(self, interval, target=None, name=None, args=None, kwargs=None, daemon=None, group=None,
+                 init=None, iargs=None, ikwargs=None):
         """Create a thread that will run a function periodically.
 
         Args:
             interval (int/float): How often to run a function in seconds.
+            target (object)[None]: Target functions to run in a separate thread.
+            name (str)[None]: Name of the new process.
+            args (tuple)[None]: Default positional arguments to pass into the given target function.
+            kwargs (dict)[None]: Default keyword arguments to pass into the given target function.
+            daemon (bool)[None]: If this process should be a daemon process. This is automatically forced to be False.
+                Non-daemon process/threads call join when python exits.
+            group (object)[None]: Not used in python multiprocessing at this time.
+            init (callable)[None]: Run this function at the start of the process. If it returns a dictionary pass the
+                dictionary as keyword arguments into the target function.
+            iargs (tuple)[None]: Positional arguments to pass into init
+            ikwargs (dict)[None]: Keyword arguments to pass into init.
         """
-        super(PeriodicThread, self).__init__(target=target, name=name, args=args, kwargs=kwargs, daemon=daemon)
+        super(PeriodicThread, self).__init__(target=target, name=name, args=args, kwargs=kwargs,
+                                             daemon=daemon, group=group, init=init, iargs=iargs, ikwargs=ikwargs)
         self.interval = interval
 
     def run(self):
         """The thread will loop through running the set _target method (default _run()). This
         method can be paused and restarted.
         """
+        args, kwargs = self.run_init()
+
+        # Allow threading._shutdown() to continue
+        lock = self.allow_shutdown()
+
         start = time.time()
         while self.alive.is_set():
-            # Run the thread method
-            self._target(*self._args, **self._kwargs)
+            # Run the thread method while protected in the lock state
+            with safe_release(lock):
+                self._target(*args, **kwargs)
             try:
                 time.sleep(self.interval - (time.time() - start))
             except ValueError:
